==> postgres-2.1.1.tar.gz: lib/src/connection_fsm.dart <==
part of postgres.connection;

abstract class _PostgreSQLConnectionState {
  PostgreSQLConnection connection;

  _PostgreSQLConnectionState onEnter() {
    return this;
  }

  _PostgreSQLConnectionState awake() {
    return this;
  }

  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    return this;
  }

  _PostgreSQLConnectionState onErrorResponse(ErrorResponseMessage message) {
    final exception = PostgreSQLException._(message.fields);

    if (exception.severity == PostgreSQLSeverity.fatal ||
        exception.severity == PostgreSQLSeverity.panic) {
      return _PostgreSQLConnectionStateClosed();
    }

    return this;
  }

  void onExit() {}
}

/*
  Closed State; starts here and ends here.
 */

class _PostgreSQLConnectionStateClosed extends _PostgreSQLConnectionState {}

/*
  Socket connected, prior to any PostgreSQL handshaking - initiates that handshaking
 */

class _PostgreSQLConnectionStateSocketConnected
    extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateSocketConnected(this.completer);

  Completer completer;

  @override
  _PostgreSQLConnectionState onEnter() {
    final startupMessage = StartupMessage(
        connection.databaseName, connection.timeZone,
        username: connection.username);

    connection._socket.add(startupMessage.asBytes());

    return this;
  }

  @override
  _PostgreSQLConnectionState onErrorResponse(ErrorResponseMessage message) {
    final exception = PostgreSQLException._(message.fields);

    completer.completeError(exception);

    return _PostgreSQLConnectionStateClosed();
  }

  @override
  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    final authMessage = message as AuthenticationMessage;

    // Pass on the pending op to subsequent stages
    if (authMessage.type == AuthenticationMessage.KindOK) {
      return _PostgreSQLConnectionStateAuthenticated(completer);
    } else if (authMessage.type == AuthenticationMessage.KindMD5Password) {
      connection._salt = authMessage.salt;

      return _PostgreSQLConnectionStateAuthenticating(completer);
    }

    completer.completeError(PostgreSQLException(
        'Unsupported authentication type ${authMessage.type}, closing connection.'));

    return _PostgreSQLConnectionStateClosed();
  }
}

/*
  Authenticating state
 */

class _PostgreSQLConnectionStateAuthenticating
    extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateAuthenticating(this.completer);

  Completer completer;

  @override
  _PostgreSQLConnectionState onEnter() {
    final authMessage = AuthMD5Message(
        connection.username, connection.password, connection._salt);

    connection._socket.add(authMessage.asBytes());

    return this;
  }

  @override
  _PostgreSQLConnectionState onErrorResponse(ErrorResponseMessage message) {
    final exception = PostgreSQLException._(message.fields);

    completer.completeError(exception);

    return _PostgreSQLConnectionStateClosed();
  }

  @override
  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    if (message is ParameterStatusMessage) {
      connection.settings[message.name] = message.value;
    } else if (message is BackendKeyMessage) {
      connection._processID = message.processID;
      connection._secretKey = message.secretKey;
    } else if (message is ReadyForQueryMessage) {
      if (message.state == ReadyForQueryMessage.StateIdle) {
        return _PostgreSQLConnectionStateIdle(openCompleter: completer);
      }
    }

    return this;
  }
}

/*
  Authenticated state
 */

class _PostgreSQLConnectionStateAuthenticated
    extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateAuthenticated(this.completer);

  Completer completer;

  @override
  _PostgreSQLConnectionState onErrorResponse(ErrorResponseMessage message) {
    final exception = PostgreSQLException._(message.fields);

    completer.completeError(exception);

    return _PostgreSQLConnectionStateClosed();
  }

  @override
  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    if (message is ParameterStatusMessage) {
      connection.settings[message.name] = message.value;
    } else if (message is BackendKeyMessage) {
      connection._processID = message.processID;
      connection._secretKey = message.secretKey;
    } else if (message is ReadyForQueryMessage) {
      if (message.state == ReadyForQueryMessage.StateIdle) {
        return _PostgreSQLConnectionStateIdle(openCompleter: completer);
      }
    }

    return this;
  }
}

/*
  Ready/idle state
 */

class _PostgreSQLConnectionStateIdle extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateIdle({this.openCompleter});

  Completer openCompleter;

  @override
  _PostgreSQLConnectionState awake() {
    final pendingQuery = connection._queue.pending;
    if (pendingQuery != null) {
      return processQuery(pendingQuery);
    }

    return this;
  }

  _PostgreSQLConnectionState processQuery(Query<dynamic> q) {
    try {
      if (q.onlyReturnAffectedRowCount) {
        q.sendSimple(connection._socket);
        return _PostgreSQLConnectionStateBusy(q);
      }

      final cached = connection._cache[q.statement];
      q.sendExtended(connection._socket, cacheQuery: cached);

      return _PostgreSQLConnectionStateBusy(q);
    } catch (e, st) {
      scheduleMicrotask(() {
        q.completeError(e, st);
        connection._transitionToState(_PostgreSQLConnectionStateIdle());
      });

      return _PostgreSQLConnectionStateDeferredFailure();
    }
  }

  @override
  _PostgreSQLConnectionState onEnter() {
    openCompleter?.complete();

    return awake();
  }

  @override
  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    return this;
  }
}

/*
  Busy state, query in progress
 */

class _PostgreSQLConnectionStateBusy extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateBusy(this.query);

  Query<dynamic> query;
  PostgreSQLException returningException;
  int rowsAffected = 0;

  @override
  _PostgreSQLConnectionState onErrorResponse(ErrorResponseMessage message) {
    // If we get an error here, then we should eat the rest of the messages
    // and we are always confirmed to get a _ReadyForQueryMessage to finish up.
    // We should only report the error once that is done.
    final exception = PostgreSQLException._(message.fields);
    returningException ??= exception;

    if (exception.severity == PostgreSQLSeverity.fatal ||
        exception.severity == PostgreSQLSeverity.panic) {
      return _PostgreSQLConnectionStateClosed();
    }

    return this;
  }

  @override
  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    // We ignore NoData, as it doesn't tell us anything we don't already know
    // or care about.

    // print("(${query.statement}) -> $message");

    if (message is ReadyForQueryMessage) {
      if (message.state == ReadyForQueryMessage.StateTransactionError) {
        query.completeError(returningException);
        return _PostgreSQLConnectionStateReadyInTransaction(
            query.transaction as _TransactionProxy);
      }

      if (returningException != null) {
        query.completeError(returningException);
      } else {
        query.complete(rowsAffected);
      }

      if (message.state == ReadyForQueryMessage.StateTransaction) {
        return _PostgreSQLConnectionStateReadyInTransaction(
            query.transaction as _TransactionProxy);
      }

      return _PostgreSQLConnectionStateIdle();
    } else if (message is CommandCompleteMessage) {
      rowsAffected = message.rowsAffected;
    } else if (message is RowDescriptionMessage) {
      query.fieldDescriptions = message.fieldDescriptions;
    } else if (message is DataRowMessage) {
      query.addRow(message.values);
    } else if (message is ParameterDescriptionMessage) {
      final validationException =
          query.validateParameters(message.parameterTypeIDs);
      if (validationException != null) {
        query.cache = null;
      }
      returningException ??= validationException;
    }

    return this;
  }
}

/* Idle Transaction State */

class _PostgreSQLConnectionStateReadyInTransaction
    extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateReadyInTransaction(this.transaction);

  _TransactionProxy transaction;

  @override
  _PostgreSQLConnectionState onEnter() {
    return awake();
  }

  @override
  _PostgreSQLConnectionState awake() {
    final pendingQuery = transaction._queue.pending;
    if (pendingQuery != null) {
      return processQuery(pendingQuery);
    }

    return this;
  }

  _PostgreSQLConnectionState processQuery(Query<dynamic> q) {
    try {
      if (q.onlyReturnAffectedRowCount) {
        q.sendSimple(connection._socket);
        return _PostgreSQLConnectionStateBusy(q);
      }

      final cached = connection._cache[q.statement];
      q.sendExtended(connection._socket, cacheQuery: cached);

      return _PostgreSQLConnectionStateBusy(q);
    } catch (e, st) {
      scheduleMicrotask(() {
        q.completeError(e, st);
      });

      return this;
    }
  }
}

/*
  Hack for deferred error
 */

class _PostgreSQLConnectionStateDeferredFailure
    extends _PostgreSQLConnectionState {}

==> postgres-2.3.2.tar.gz: lib/src/connection_fsm.dart <==
part of postgres.connection;

abstract class _PostgreSQLConnectionState {
  PostgreSQLConnection? connection;

  _PostgreSQLConnectionState onEnter() {
    return this;
  }

  _PostgreSQLConnectionState awake() {
    return this;
  }

  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    return this;
  }

  _PostgreSQLConnectionState onErrorResponse(ErrorResponseMessage message) {
    final exception = PostgreSQLException._(message.fields);

    if (exception.severity == PostgreSQLSeverity.fatal ||
        exception.severity == PostgreSQLSeverity.panic) {
      return _PostgreSQLConnectionStateClosed();
    }

    return this;
  }

  void onExit() {}
}

/*
  Closed State; starts here and ends here.
 */

class _PostgreSQLConnectionStateClosed extends _PostgreSQLConnectionState {}

/*
  Socket connected, prior to any PostgreSQL handshaking - initiates that handshaking
 */

class _PostgreSQLConnectionStateSocketConnected
    extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateSocketConnected(this.completer);

  Completer completer;

  @override
  _PostgreSQLConnectionState onEnter() {
    final startupMessage = StartupMessage(
        connection!.databaseName, connection!.timeZone,
        username: connection!.username);

    connection!._socket!.add(startupMessage.asBytes());

    return this;
  }

  @override
  _PostgreSQLConnectionState onErrorResponse(ErrorResponseMessage message) {
    final exception = PostgreSQLException._(message.fields);

    completer.completeError(exception);

    return _PostgreSQLConnectionStateClosed();
  }

  @override
  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    final authMessage = message as AuthenticationMessage;

    // Pass on the pending op to subsequent stages
    if (authMessage.type == AuthenticationMessage.KindOK) {
      return _PostgreSQLConnectionStateAuthenticated(completer);
    } else if (authMessage.type == AuthenticationMessage.KindMD5Password) {
      connection!._salt = authMessage.salt;

      return _PostgreSQLConnectionStateAuthenticating(completer);
    }

    completer.completeError(PostgreSQLException(
        'Unsupported authentication type ${authMessage.type}, closing connection.'));

    return _PostgreSQLConnectionStateClosed();
  }
}

/*
  Authenticating state
 */

class _PostgreSQLConnectionStateAuthenticating
    extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateAuthenticating(this.completer);

  Completer completer;

  @override
  _PostgreSQLConnectionState onEnter() {
    final authMessage = AuthMD5Message(
        connection!.username!, connection!.password!, connection!._salt);

    connection!._socket!.add(authMessage.asBytes());

    return this;
  }

  @override
  _PostgreSQLConnectionState onErrorResponse(ErrorResponseMessage message) {
    final exception = PostgreSQLException._(message.fields);

    completer.completeError(exception);

    return _PostgreSQLConnectionStateClosed();
  }

  @override
  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    if (message is ParameterStatusMessage) {
      connection!.settings[message.name] = message.value;
    } else if (message is BackendKeyMessage) {
      connection!._processID = message.processID;
      connection!._secretKey = message.secretKey;
    } else if (message is ReadyForQueryMessage) {
      if (message.state == ReadyForQueryMessage.StateIdle) {
        return _PostgreSQLConnectionStateIdle(openCompleter: completer);
      }
    }

    return this;
  }
}

/*
  Authenticated state
 */

class _PostgreSQLConnectionStateAuthenticated
    extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateAuthenticated(this.completer);

  Completer completer;

  @override
  _PostgreSQLConnectionState onErrorResponse(ErrorResponseMessage message) {
    final exception = PostgreSQLException._(message.fields);

    completer.completeError(exception);

    return _PostgreSQLConnectionStateClosed();
  }

  @override
  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    if (message is ParameterStatusMessage) {
      connection!.settings[message.name] = message.value;
    } else if (message is BackendKeyMessage) {
      connection!._processID = message.processID;
      connection!._secretKey = message.secretKey;
    } else if (message is ReadyForQueryMessage) {
      if (message.state == ReadyForQueryMessage.StateIdle) {
        return _PostgreSQLConnectionStateIdle(openCompleter: completer);
      }
    }

    return this;
  }
}

/*
  Ready/idle state
 */

class _PostgreSQLConnectionStateIdle extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateIdle({this.openCompleter});

  Completer? openCompleter;

  @override
  _PostgreSQLConnectionState awake() {
    final pendingQuery = connection!._queue.pending;
    if (pendingQuery != null) {
      return processQuery(pendingQuery);
    }

    return this;
  }

  _PostgreSQLConnectionState processQuery(Query<dynamic> q) {
    try {
      if (q.onlyReturnAffectedRowCount) {
        q.sendSimple(connection!._socket!);
        return _PostgreSQLConnectionStateBusy(q);
      }

      final cached = connection!._cache[q.statement];
      q.sendExtended(connection!._socket!, cacheQuery: cached);

      return _PostgreSQLConnectionStateBusy(q);
    } catch (e, st) {
      scheduleMicrotask(() {
        q.completeError(e, st);
        connection!._transitionToState(_PostgreSQLConnectionStateIdle());
      });

      return _PostgreSQLConnectionStateDeferredFailure();
    }
  }

  @override
  _PostgreSQLConnectionState onEnter() {
    openCompleter?.complete();

    return awake();
  }

  @override
  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    return this;
  }
}

/*
  Busy state, query in progress
 */

class _PostgreSQLConnectionStateBusy extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateBusy(this.query);

  Query<dynamic> query;
  PostgreSQLException? returningException;
  int rowsAffected = 0;

  @override
  _PostgreSQLConnectionState onErrorResponse(ErrorResponseMessage message) {
    // If we get an error here, then we should eat the rest of the messages
    // and we are always confirmed to get a _ReadyForQueryMessage to finish up.
    // We should only report the error once that is done.
    final exception = PostgreSQLException._(message.fields);
    returningException ??= exception;

    if (exception.severity == PostgreSQLSeverity.fatal ||
        exception.severity == PostgreSQLSeverity.panic) {
      return _PostgreSQLConnectionStateClosed();
    }

    return this;
  }

  @override
  _PostgreSQLConnectionState onMessage(ServerMessage message) {
    // We ignore NoData, as it doesn't tell us anything we don't already know
    // or care about.

    // print("(${query.statement}) -> $message");

    if (message is ReadyForQueryMessage) {
      if (message.state == ReadyForQueryMessage.StateTransactionError) {
        query.completeError(returningException!);
        return _PostgreSQLConnectionStateReadyInTransaction(
            query.transaction as _TransactionProxy);
      }
      if (returningException != null) {
        query.completeError(returningException!);
      } else {
        query.complete(rowsAffected);
      }

      if (message.state == ReadyForQueryMessage.StateTransaction) {
        return _PostgreSQLConnectionStateReadyInTransaction(
            query.transaction as _TransactionProxy);
      }

      return _PostgreSQLConnectionStateIdle();
    } else if (message is CommandCompleteMessage) {
      rowsAffected = message.rowsAffected;
    } else if (message is RowDescriptionMessage) {
      query.fieldDescriptions = message.fieldDescriptions;
    } else if (message is DataRowMessage) {
      query.addRow(message.values);
    } else if (message is ParameterDescriptionMessage) {
      final validationException =
          query.validateParameters(message.parameterTypeIDs);
      if (validationException != null) {
        query.cache = null;
      }
      returningException ??= validationException;
    }

    return this;
  }
}

/* Idle Transaction State */

class _PostgreSQLConnectionStateReadyInTransaction
    extends _PostgreSQLConnectionState {
  _PostgreSQLConnectionStateReadyInTransaction(this.transaction);

  _TransactionProxy transaction;

  @override
  _PostgreSQLConnectionState onEnter() {
    return awake();
  }

  @override
  _PostgreSQLConnectionState awake() {
    final pendingQuery = transaction._queue.pending;
    if (pendingQuery != null) {
      return processQuery(pendingQuery);
    }

    return this;
  }

  _PostgreSQLConnectionState processQuery(Query<dynamic> q) {
    try {
      if (q.onlyReturnAffectedRowCount) {
        q.sendSimple(connection!._socket!);
        return _PostgreSQLConnectionStateBusy(q);
      }

      final cached = connection!._cache[q.statement];
      q.sendExtended(connection!._socket!, cacheQuery: cached);

      return _PostgreSQLConnectionStateBusy(q);
    } catch (e, st) {
      scheduleMicrotask(() {
        q.completeError(e, st);
      });

      return this;
    }
  }
}

/*
  Hack for deferred error
 */

class _PostgreSQLConnectionStateDeferredFailure
    extends _PostgreSQLConnectionState {}
